@@ -14,12 +14,17 @@ import (
 
 // toggleWrapMode toggles wrapMode each time it is called.
 func (root *Root) toggleWrapMode() {
-	root.Doc.WrapMode = !root.Doc.WrapMode
-	root.Doc.x = 0
-	if root.Doc.ColumnMode {
-		root.Doc.x = root.columnModeX()
+	m := root.Doc
+	m.WrapMode = !m.WrapMode
+	m.x = 0
+	if m.ColumnMode {
+		x, err := root.columnX(m.columnCursor)
+		if err != nil {
+			x = 0
+		}
+		m.x = x
 	}
-	root.setMessagef("Set WrapMode %t", root.Doc.WrapMode)
+	root.setMessagef("Set WrapMode %t", m.WrapMode)
 }
 
 // toggleColumnMode toggles ColumnMode each time it is called.
@@ -333,8 +338,7 @@ func (root *Root) setViewMode(modeName string) {
 	}
 
 	root.Doc.general = mergeGeneral(root.Doc.general, c)
-	root.Doc.setSectionDelimiter(root.Doc.SectionDelimiter)
-	root.Doc.setMultiColorWords(root.Doc.MultiColorWords)
+	root.Doc.regexpCompile()
 	root.Doc.ClearCache()
 	root.ViewSync()
 	root.setMessagef("Set mode %s", modeName)
@@ -342,7 +346,7 @@ func (root *Root) setViewMode(modeName string) {
 
 // setDelimiter sets the delimiter string.
 func (root *Root) setDelimiter(input string) {
-	root.Doc.ColumnDelimiter = input
+	root.Doc.setDelimiter(input)
 	root.setMessagef("Set delimiter %s", input)
 }
 
