@@ -62,7 +62,7 @@ func (root *Root) drawHeader() int {
 			break
 		}
 
-		lc := m.getContents(lY, m.TabWidth)
+		lc, _ := m.getContents(lY, m.TabWidth)
 		lineStr, posCV := m.getContentsStr(lY, lc)
 		root.lines[hy] = line{
 			number: lY,
@@ -92,18 +92,18 @@ func (root *Root) drawBody(lX int, lY int) (int, int) {
 	m := root.Doc
 
 	wrapNum := root.numOfWrap(lX, lY)
-	markStyleWidth := min(root.vWidth, root.Doc.general.MarkStyleWidth)
 	root.Doc.lastContentsNum = -1
 
 	// lc, lineStr, byteMap store the previous value.
 	// Because it may be a continuation from the previous line in wrap mode.
 	lastLN := -1
 	var lc contents
+	var valid bool
 	var lineStr string
 	var posCV map[int]int
 	for y := root.headerLen; y < root.vHight-statusLine; y++ {
 		if lastLN != lY {
-			lc = m.getContents(lY, m.TabWidth)
+			lc, valid = m.getContents(lY, m.TabWidth)
 			lineStr, posCV = m.getContentsStr(lY, lc)
 			root.bodyStyle(lc, root.StyleBody)
 			lastLN = lY
@@ -114,22 +114,21 @@ func (root *Root) drawBody(lX int, lY int) (int, int) {
 			wrap:   wrapNum,
 		}
 
-		if root.Doc.PlainMode {
-			root.plainStyle(lc)
+		if valid {
+			root.styleContent(lY, lc, lineStr, posCV)
 		}
-		root.columnHighlight(lc, lineStr, posCV)
-		root.multiColorHighlight(lc, lineStr, posCV)
-		root.searchHighlight(lY, lc, lineStr, posCV)
-		root.drawLineNumber(lY, y)
 
-		currentY := lY
+		if valid {
+			root.drawLineNumber(lY, y)
+		} else {
+			root.blankLineNumber(y)
+		}
+
+		currentlY := lY
 		lX, lY = root.drawLine(y, lX, lY, lc)
 
-		root.alternateRowsStyle(currentY, y)
-		root.markStyle(currentY, y, markStyleWidth)
-		root.sectionLineHighlight(y, lineStr)
-		if root.Doc.JumpTarget != 0 && root.headerLen+root.Doc.JumpTarget == y {
-			root.lineStyle(y, root.StyleJumpTargetLine)
+		if valid {
+			root.styleLine(currentlY, y, lc, lineStr)
 		}
 
 		if lX > 0 {
@@ -142,6 +141,25 @@ func (root *Root) drawBody(lX int, lY int) (int, int) {
 	return lX, lY
 }
 
+func (root *Root) styleContent(lY int, lc contents, lineStr string, posCV map[int]int) {
+	if root.Doc.PlainMode {
+		root.plainStyle(lc)
+	}
+	root.columnHighlight(lc, lineStr, posCV)
+	root.multiColorHighlight(lc, lineStr, posCV)
+	root.searchHighlight(lY, lc, lineStr, posCV)
+}
+
+func (root *Root) styleLine(currentY int, y int, lc contents, lineStr string) {
+	root.alternateRowsStyle(currentY, y)
+	markStyleWidth := min(root.vWidth, root.Doc.general.MarkStyleWidth)
+	root.markStyle(currentY, y, markStyleWidth)
+	root.sectionLineHighlight(y, lineStr)
+	if root.Doc.JumpTarget != 0 && root.headerLen+root.Doc.JumpTarget == y {
+		root.lineStyle(y, root.StyleJumpTargetLine)
+	}
+}
+
 // drawWrapLine wraps and draws the contents and returns the next drawing position.
 func (root *Root) drawLine(y int, lX int, lY int, lc contents) (int, int) {
 	if root.Doc.WrapMode {
@@ -267,20 +285,20 @@ func (root *Root) drawLineNumber(lY int, y int) {
 
 // columnHighlight applies the style of the column highlight.
 func (root *Root) columnHighlight(lc contents, str string, posCV map[int]int) {
-	if !root.Doc.ColumnMode {
+	m := root.Doc
+	if !m.ColumnMode {
 		return
 	}
 
 	numC := len(root.StyleColumnRainbow)
-	delm := root.Doc.ColumnDelimiter
 	c := 0
 	start := 0
-	idxs := allIndex(str, delm)
+	idxs := allIndex(str, m.ColumnDelimiter, m.ColumnDelimiterReg)
 	for _, idx := range idxs {
-		if root.Doc.ColumnRainbow {
+		if m.ColumnRainbow {
 			RangeStyle(lc, posCV[start], posCV[idx[0]], root.StyleColumnRainbow[c%numC])
 		}
-		if c == root.Doc.columnNum {
+		if c == m.columnCursor {
 			RangeStyle(lc, posCV[start], posCV[idx[0]], root.StyleColumnHighlight)
 		}
 		start = idx[1]
@@ -289,10 +307,10 @@ func (root *Root) columnHighlight(lc contents, str string, posCV map[int]int) {
 		}
 	}
 	if start < len(str) {
-		if root.Doc.ColumnRainbow {
+		if m.ColumnRainbow {
 			RangeStyle(lc, posCV[start], posCV[len(str)], root.StyleColumnRainbow[c%numC])
 		}
-		if c == root.Doc.columnNum && start != 0 {
+		if c == m.columnCursor && start != 0 {
 			RangeStyle(lc, posCV[start], posCV[len(str)], root.StyleColumnHighlight)
 		}
 	}
@@ -327,7 +345,7 @@ func (root *Root) lineStyle(y int, s OVStyle) {
 // markStyle applies the style from the left edge to the specified width.
 func (root *Root) markStyle(lY int, y int, width int) {
 	m := root.Doc
-	if containsInt(m.marked, lY) {
+	if contains(m.marked, lY) {
 		for x := 0; x < width; x++ {
 			r, c, style, _ := root.GetContent(x, y)
 			root.SetContent(x, y, r, c, applyStyle(style, root.StyleMarkLine))
