@@ -2,6 +2,7 @@ package oviewer
 
 import (
 	"fmt"
+	"io"
 	"strings"
 
 	"code.rocketnine.space/tslocum/cbind"
@@ -74,6 +75,8 @@ const (
 	inputCaseSensitive = "input_casesensitive"
 	inputIncSearch     = "input_incsearch"
 	inputRegexpSearch  = "input_regexp_search"
+	inputPrevious      = "input_previous"
+	inputNext          = "input_next"
 )
 
 // handlers returns a map of the action's handlers.
@@ -110,7 +113,7 @@ func (root *Root) handlers() map[string]func() {
 		actionMoveHfRight:    root.moveHfRight,
 		actionSection:        root.setSectionDelimiterMode,
 		actionSectionStart:   root.setSectionStartMode,
-		actionNextSection:    root.nextSction,
+		actionNextSection:    root.nextSection,
 		actionPrevSection:    root.prevSection,
 		actionLastSection:    root.lastSection,
 		actionMoveMark:       root.markNext,
@@ -142,6 +145,8 @@ func (root *Root) handlers() map[string]func() {
 		inputCaseSensitive: root.inputCaseSensitive,
 		inputIncSearch:     root.inputIncSearch,
 		inputRegexpSearch:  root.inputRegexpSearch,
+		inputPrevious:      root.inputPrevious,
+		inputNext:          root.inputNext,
 	}
 }
 
@@ -214,6 +219,8 @@ func defaultKeyBinds() KeyBind {
 		inputCaseSensitive: {"alt+c"},
 		inputIncSearch:     {"alt+i"},
 		inputRegexpSearch:  {"alt+r"},
+		inputPrevious:      {"Up"},
+		inputNext:          {"Down"},
 	}
 }
 
@@ -248,7 +255,7 @@ func (k KeyBind) String() string {
 	k.writeKeyBind(&b, actionMoveRight, "scroll to right")
 	k.writeKeyBind(&b, actionMoveHfLeft, "scroll left half screen")
 	k.writeKeyBind(&b, actionMoveHfRight, "scroll right half screen")
-	k.writeKeyBind(&b, actionGoLine, "go to line(input number)")
+	k.writeKeyBind(&b, actionGoLine, "go to line(input number and `.n` and `n%` allowed)")
 
 	fmt.Fprint(&b, "\n\tMove document\n")
 	fmt.Fprint(&b, "\n")
@@ -278,7 +285,7 @@ func (k KeyBind) String() string {
 	k.writeKeyBind(&b, actionRainbow, "column rainbow toggle")
 	k.writeKeyBind(&b, actionAlternate, "alternate rows of style toggle")
 	k.writeKeyBind(&b, actionLineNumMode, "line number toggle")
-	k.writeKeyBind(&b, actionPlain, "original decoration toggle")
+	k.writeKeyBind(&b, actionPlain, "original decoration toggle(plain)")
 
 	fmt.Fprint(&b, "\n\tChange Display with Input\n")
 	fmt.Fprint(&b, "\n")
@@ -288,7 +295,7 @@ func (k KeyBind) String() string {
 	k.writeKeyBind(&b, actionSkipLines, "number of skip lines")
 	k.writeKeyBind(&b, actionTabWidth, "TAB width")
 	k.writeKeyBind(&b, actionMultiColor, "multi color highlight")
-	k.writeKeyBind(&b, actionJumpTarget, "jump target")
+	k.writeKeyBind(&b, actionJumpTarget, "jump target(`.n` and `n%` allowed)")
 
 	fmt.Fprint(&b, "\n\tSection\n")
 	fmt.Fprint(&b, "\n")
@@ -311,27 +318,32 @@ func (k KeyBind) String() string {
 	k.writeKeyBind(&b, inputCaseSensitive, "case-sensitive toggle")
 	k.writeKeyBind(&b, inputRegexpSearch, "regular expression search toggle")
 	k.writeKeyBind(&b, inputIncSearch, "incremental search toggle")
+	k.writeKeyBind(&b, inputPrevious, "previous candidate")
+	k.writeKeyBind(&b, inputNext, "next candidate")
 	return b.String()
 }
 
+func (k KeyBind) writeKeyBind(w io.Writer, action string, detail string) {
+	fmt.Fprintf(w, " %-28s * %s\n", "["+strings.Join(k[action], "], [")+"]", detail)
+}
+
 // GetKeyBinds returns the current key mapping.
 func GetKeyBinds(config Config) KeyBind {
 	keyBind := make(map[string][]string)
+
 	if strings.ToLower(config.DefaultKeyBind) != "disable" {
 		keyBind = defaultKeyBinds()
 	}
-	keyBind = updateKeyBind(keyBind, config.Keybind)
-	return keyBind
-}
 
-func updateKeyBind(keyBind KeyBind, bind KeyBind) KeyBind {
-	for k, v := range bind {
+	// Overwrite with config file.
+	for k, v := range config.Keybind {
 		keyBind[k] = v
 	}
 	return keyBind
 }
 
-func (root *Root) setKeyBind(keyBind KeyBind) error {
+// setHandlers sets keys to action handlers.
+func (root *Root) setHandlers(keyBind KeyBind) error {
 	c := root.keyConfig
 	in := root.inputKeyConfig
 
@@ -356,6 +368,7 @@ func (root *Root) setKeyBind(keyBind KeyBind) error {
 	return nil
 }
 
+// setHandler sets multiple keys in one action handler.
 func setHandler(c *cbind.Configuration, name string, keys []string, handler func()) error {
 	for _, k := range keys {
 		mod, key, ch, err := cbind.Decode(k)
@@ -375,6 +388,7 @@ func setHandler(c *cbind.Configuration, name string, keys []string, handler func
 	return nil
 }
 
+// wrapEventHandler is a wrapper for matching func types.
 func wrapEventHandler(f func()) func(_ *tcell.EventKey) *tcell.EventKey {
 	return func(_ *tcell.EventKey) *tcell.EventKey {
 		f()
@@ -382,6 +396,7 @@ func wrapEventHandler(f func()) func(_ *tcell.EventKey) *tcell.EventKey {
 	}
 }
 
+// keyCapture does the actual key action.
 func (root *Root) keyCapture(ev *tcell.EventKey) bool {
 	root.keyConfig.Capture(ev)
 	return true
