@@ -21,18 +21,12 @@ func (m *Document) moveTop() {
 // Go to the top line.
 // Called from a EventKey.
 func (root *Root) moveTop() {
-	root.resetSelect()
-	defer root.releaseEventBuffer()
-
 	root.Doc.moveTop()
 }
 
 // Go to the bottom line.
 // Called from a EventKey.
 func (root *Root) moveBottom() {
-	root.resetSelect()
-	defer root.releaseEventBuffer()
-
 	root.Doc.topLX, root.Doc.topLN = root.bottomLineNum(root.Doc.BufEndNum())
 }
 
@@ -270,7 +264,8 @@ func (root *Root) moveDown() {
 	root.limitMoveDown(m.topLX, num)
 }
 
-func (root *Root) nextSction() {
+// nextSection moves down to the next section's delimiter.
+func (root *Root) nextSection() {
 	// Move by page, if there is no section delimiter.
 	if root.Doc.SectionDelimiter == "" {
 		root.movePgDn()
@@ -295,6 +290,7 @@ func (root *Root) nextSction() {
 	root.Doc.moveLine((n - m.firstLine()) + m.SectionStartPosition)
 }
 
+// prevSection moves up to the delimiter of the previous section.
 func (root *Root) prevSection() {
 	// Move by page, if there is no section delimiter.
 	if root.Doc.SectionDelimiter == "" {
@@ -320,6 +316,7 @@ func (root *Root) prevSection() {
 	m.moveLine(n)
 }
 
+// lastSection moves to the last section.
 func (root *Root) lastSection() {
 	root.resetSelect()
 
@@ -345,19 +342,26 @@ func (root *Root) moveLeft() {
 	defer root.releaseEventBuffer()
 
 	m := root.Doc
-	if m.ColumnMode {
-		if m.columnNum > 0 {
-			m.columnNum--
-			m.x = root.columnModeX()
+	if !m.ColumnMode {
+		if m.WrapMode {
+			return
+		}
+		m.x--
+		if m.x < root.minStartX {
+			m.x = root.minStartX
 		}
 		return
 	}
-	if m.WrapMode {
-		return
-	}
-	m.x--
-	if m.x < root.minStartX {
-		m.x = root.minStartX
+
+	if m.columnCursor > 0 {
+		cursor := m.columnCursor - 1
+		x, err := root.columnX(cursor)
+		if err != nil {
+			root.debugMessage(err.Error())
+			return
+		}
+		m.x = x
+		m.columnCursor = cursor
 	}
 }
 
@@ -368,21 +372,32 @@ func (root *Root) moveRight() {
 	defer root.releaseEventBuffer()
 
 	m := root.Doc
-	if m.ColumnMode {
-		m.columnNum++
-		m.x = root.columnModeX()
+	if !m.ColumnMode {
+		if m.WrapMode {
+			return
+		}
+		m.x++
 		return
 	}
-	if m.WrapMode {
+
+	cursor := m.columnCursor + 1
+	x, err := root.columnX(cursor)
+	if err != nil {
+		root.debugMessage(err.Error())
 		return
 	}
-	m.x++
+	m.x = x
+	m.columnCursor = cursor
 }
 
-// columnModeX returns the actual x from m.columnNum.
-func (root *Root) columnModeX() int {
+// columnX returns the columnCursor and actual x from columnCursor.
+func (root *Root) columnX(cursor int) (int, error) {
 	m := root.Doc
-	maxNum := 0
+	if cursor <= 0 {
+		return 0, nil
+	}
+
+	maxCursor := 0
 	// m.firstLine()+10 = Maximum columnMode target.
 	for i := 0; i < m.firstLine()+10; i++ {
 		lc, err := m.contentsLN(m.topLN+m.firstLine()+i, m.TabWidth)
@@ -390,36 +405,36 @@ func (root *Root) columnModeX() int {
 			continue
 		}
 		lineStr, posCV := ContentsToStr(lc)
-		idxs := allIndex(lineStr, m.ColumnDelimiter)
-		maxNum = max(maxNum, len(idxs))
-		if len(idxs) < m.columnNum {
+		idxs := allIndex(lineStr, m.ColumnDelimiter, m.ColumnDelimiterReg)
+		maxCursor = max(maxCursor, len(idxs))
+		if len(idxs) < cursor {
 			continue
 		}
 
 		sx, ex := 0, 0
-		if len(idxs) == m.columnNum {
-			pos := idxs[m.columnNum-1]
+		if len(idxs) == cursor {
+			// right edge.
+			pos := idxs[cursor-1]
 			sx = posCV[pos[1]+len(m.ColumnDelimiter)]
 			ex = posCV[len(lineStr)]
 		} else {
-			pos := idxs[m.columnNum]
+			pos := idxs[cursor]
 			sx = posCV[pos[0]]
 			ex = posCV[pos[1]]
 		}
-		if root.vWidth > ex {
-			return 0
+		if root.vWidth > ex { // don't scroll.
+			return 0, nil
 		}
 		if ex-root.vWidth > 0 {
-			return ex - root.vWidth
+			return ex - root.vWidth, nil
 		}
-		return sx
+		return sx, nil
 	}
-	if maxNum > 0 {
-		m.columnNum--
-		return m.x
+
+	if maxCursor > 0 {
+		return m.x, ErrNoColumn
 	}
-	m.columnNum = 0
-	return 0
+	return 0, ErrNoDelimiter
 }
 
 // Move to the left by half a screen.
