@@ -129,6 +129,8 @@ type general struct {
 	WrapMode bool
 	// ColumnDelimiter is a column delimiter.
 	ColumnDelimiter string
+	// ColumnDelimiterReg is a compiled regular expression of ColumnDelimiter.
+	ColumnDelimiterReg *regexp.Regexp
 	// FollowMode is the follow mode.
 	FollowMode bool
 	// FollowAll is a follow mode for all documents.
@@ -251,6 +253,24 @@ type OVStyle struct {
 	Underline bool
 	// If true, add strikethrough.
 	StrikeThrough bool
+	// If true, add overline (not yet supported).
+	OverLine bool
+	// If true, sub blink.
+	UnBlink bool
+	// If true, sub bold.
+	UnBold bool
+	// If true, sub dim.
+	UnDim bool
+	// If true, sub italic.
+	UnItalic bool
+	// If true, sub reverse.
+	UnReverse bool
+	// If true, sub underline.
+	UnUnderline bool
+	// If true, sub strikethrough.
+	UnStrikeThrough bool
+	// if true, sub underline (not yet supported).
+	UnOverLine bool
 }
 
 var (
@@ -309,6 +329,10 @@ var (
 	ErrSignalCatch = errors.New("signal catch")
 	// ErrAlreadyClose indicates that it is already closed.
 	ErrAlreadyClose = errors.New("already closed")
+	// ErrNoColumn indicates that cusror specified a nonexistent column.
+	ErrNoColumn = errors.New("no column")
+	// ErrNoDelimiter indicates that the line containing the delimiter could not be found.
+	ErrNoDelimiter = errors.New("no delimiter")
 )
 
 // This is a function of tcell.NewScreen but can be replaced with mock.
@@ -472,7 +496,7 @@ func (root *Root) SetWatcher(watcher *fsnotify.Watcher) {
 
 func (root *Root) setKeyConfig() (map[string][]string, error) {
 	keyBind := GetKeyBinds(root.Config)
-	if err := root.setKeyBind(keyBind); err != nil {
+	if err := root.setHandlers(keyBind); err != nil {
 		return nil, err
 	}
 
@@ -643,25 +667,46 @@ func applyStyle(style tcell.Style, s OVStyle) tcell.Style {
 		style = style.Foreground(tcell.GetColor(s.Foreground))
 	}
 	if s.Blink {
-		style = style.Blink(s.Blink)
+		style = style.Blink(true)
 	}
 	if s.Bold {
-		style = style.Bold(s.Bold)
+		style = style.Bold(true)
 	}
 	if s.Dim {
-		style = style.Dim(s.Dim)
+		style = style.Dim(true)
 	}
 	if s.Italic {
-		style = style.Italic(s.Italic)
+		style = style.Italic(true)
 	}
 	if s.Reverse {
-		style = style.Reverse(s.Reverse)
+		style = style.Reverse(true)
 	}
 	if s.Underline {
-		style = style.Underline(s.Underline)
+		style = style.Underline(true)
 	}
 	if s.StrikeThrough {
-		style = style.StrikeThrough(s.StrikeThrough)
+		style = style.StrikeThrough(true)
+	}
+	if s.UnBlink {
+		style = style.Blink(false)
+	}
+	if s.UnBold {
+		style = style.Bold(false)
+	}
+	if s.UnDim {
+		style = style.Dim(false)
+	}
+	if s.UnItalic {
+		style = style.Italic(false)
+	}
+	if s.UnReverse {
+		style = style.Reverse(false)
+	}
+	if s.UnUnderline {
+		style = style.Underline(false)
+	}
+	if s.UnStrikeThrough {
+		style = style.StrikeThrough(false)
 	}
 	return style
 }
